"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSwaggerPathsFromControllers = exports.buildRoutesFromController = void 0;
require("reflect-metadata");
const decorators_1 = require("./decorators");
function buildRoutesFromController(controller) {
    const controllerProps = Reflect.getMetadata(decorators_1.controllerMetaKey, controller);
    const methodMappings = Reflect.getMetadata(decorators_1.methodsMetaKey, controller);
    if (!controllerProps || !methodMappings)
        throw new Error('Invalid controller provided');
    const rawBasePath = controllerProps.basePath;
    if (!rawBasePath)
        throw new Error('Provided object is not a valid controller');
    const basePath = rawBasePath.replace(/\/$/, '');
    const routes = {};
    for (const { method, path: rawResourcePath, propertyKey } of Object.values(methodMappings)) {
        const resourcePath = rawResourcePath ? rawResourcePath.replace(/(^)([^/])/, '/$2').replace(/\/$/, '') : '';
        const path = basePath + resourcePath;
        routes[path] = routes[path] || { path };
        switch (method) {
            case 'GET':
                routes[path].get = controller[propertyKey];
                break;
            case 'POST':
                routes[path].post = controller[propertyKey];
                break;
            case 'PUT':
                routes[path].put = controller[propertyKey];
                break;
            case 'DELETE':
                routes[path].delete = controller[propertyKey];
                break;
        }
    }
    return Object.values(routes);
}
exports.buildRoutesFromController = buildRoutesFromController;
function buildSwaggerPathsFromControllers(controllers) {
    const paths = {};
    for (const controller of controllers) {
        const controllerProps = Reflect.getMetadata(decorators_1.controllerMetaKey, controller);
        const methodMappings = Reflect.getMetadata(decorators_1.methodsMetaKey, controller);
        if (!controllerProps || !methodMappings)
            throw new Error('Invalid controller provided');
        const rawBasePath = controllerProps.basePath;
        const tags = controllerProps.tags;
        if (!rawBasePath)
            throw new Error('Provided object is not a valid controller');
        const basePath = rawBasePath.replace(/\/$/, '');
        for (const { method, path: rawResourcePath, parameters } of Object.values(methodMappings)) {
            const resourcePath = rawResourcePath ? rawResourcePath.replace(/(^)([^/])/, '/$2').replace(/\/$/, '') : '';
            const path = basePath + resourcePath;
            const lowerCaseMethod = method.toLowerCase();
            paths[path] = paths[path] || {};
            paths[path][lowerCaseMethod] = {
                tags,
                parameters
            };
        }
    }
    return paths;
}
exports.buildSwaggerPathsFromControllers = buildSwaggerPathsFromControllers;
